//---------------------------------------------------------------------------
// TestHarvestInterface
//---------------------------------------------------------------------------
#if !defined(TestHarvestInterface_H)
  #define TestHarvestInterface_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the class clHarvestInterface.
*/
class clTestHarvestInterface : public CppUnit::TestFixture {

  public:

  /**
  * Performs all the tests from one function.
  */
  void TestHarvestInterface();

  protected:

  /**
  * Tests normal processing, run 1.  WriteXMLFile1() does setup.  Then 55 trees
  * are created.  Two timesteps are run.  In the first, the following trees 
  * should be dead:  2, 3, 9, 15, 19, 20, 21, 22, 34, 48, 50.  In the second
  * timestep, the following trees should be dead:  1,4, 30, 31.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessingRun1(clSimManager *p_oSimManager);
  
  /**
  * Tests normal processing, run 2.  This tests extra columns desired in the
  * files.  WriteXMLFile2() does setup.  Then 55 trees
  * are created.  Three timesteps are run.  In the first, the following trees 
  * should be dead:  2, 3, 9, 15, 19, 20, 21, 22, 34, 48, 50.  In the second
  * timestep, all remaining trees are cut.  The third timestep tests the 
  * situation when there are no trees.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessingRun2(clSimManager *p_oSimManager);
  
  /**
  * Tests normal processing, run 3.  This tests extra columns desired in the
  * files, and new tree members to be added and updated.  WriteXMLFile3() does 
  * setup.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessingRun3(clSimManager *p_oSimManager);
  
  /**
  * Tests normal processing, run 4.  This tests applying the behavior to only
  * a subset of trees.  WriteXMLFile4() does setup.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessingRun4(clSimManager *p_oSimManager);
  
  /**
  * Tests normal processing, run 5.  This tests batch running.  WriteXMLBatch1()
  * runs WriteXMLFile5() for setup.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessingRun5(clSimManager *p_oSimManager);
  
  /**
  * Tests normal processing, run 6.  This tests batch running with no extra
  * parameters file.  WriteXMLBatch1() runs WriteXMLFile6() for setup.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestNormalProcessingRun6(clSimManager *p_oSimManager);
  
  /**
  * Tests error processing.  This uses WriteXMLErrorFileX() to introduce errors
  * and make sure that they are caught.
  * @param p_oSimManager Sim Manager object to call for runs.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file for normal processing run 1.
  * @return Filename written.
  */
  char* WriteXMLFile1();
  
  /**
  * Writes a parameter file for normal processing run 2.
  * @return Filename written.
  */
  char* WriteXMLFile2();
  
  /**
  * Writes a parameter file for normal processing run 3.
  * @return Filename written.
  */
  char* WriteXMLFile3();
  
  /**
  * Writes a parameter file for normal processing run 4.
  * @return Filename written.
  */
  char* WriteXMLFile4();
  
  /**
  * Writes a parameter file for normal processing run 5.
  * @return Filename written.
  */
  char* WriteXMLFile5();
  
  /**
  * Writes a parameter file for normal processing run 6.
  * @return Filename written.
  */
  char* WriteXMLFile6();
  
  /**
  * Writes batch file 1, for normal processing run 5.
  * @return Filename written.
  */
  char* WriteXMLBatch1();
  
  /**
  * Writes a batch file to provoke an error in the length of the batch
  * parameters file. 
  * @return Filename written.
  */
  char* WriteXMLErrorBatch1();
  
  /**
  * Writes a parameters file, for normal processing run 5.
  * @return Filename written.
  */
  char* WriteAllParamsFile();
  
  /**
  * Writes an error file where the harvest executable isn't where it's supposed
  * to be.
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes an error file where the harvest period is negative.
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();
  
  /**
  * Writes an error file where requested tree attributes don't exist at all.
  * @return Filename written.
  */
  char* WriteXMLErrorFile3();
  
  /**
  * Writes an error file where requested tree attributes don't exist for all
  * species.
  * @return Filename written.
  */
  char* WriteXMLErrorFile4();
  
  /**
  * Writes an error file where the harvest return file will not exist.
  * @return Filename written.
  */
  char* WriteXMLErrorFile5();
  
  /**
  * Writes an error file where the update return file will not exist.
  * @return Filename written.
  */
  char* WriteXMLErrorFile6();
  
  /**
  * Writes an error file where a tree to harvest will not exist.
  * @return Filename written.
  */
  char* WriteXMLErrorFile7();
  
  /**
  * Writes an error file where a tree to update will not exist.
  * @return Filename written.
  */
  char* WriteXMLErrorFile8();
  
  /**
  * Writes an error file where the parameter batch file is missing.
  * @return Filename written.
  */
  char* WriteXMLErrorFile9();

  /**
  * Writes the tree portions of the test parameter files, from &lt;trees&gt; to
  * &lt;/trees&gt;.
  * @param oOut File stream to write to.
  */
  void WriteTrees(std::fstream &oOut);
  
  /**
  * Writes the allometry portions of the test parameter files, from 
  * &lt;allometry&gt; to &lt;/allometry&gt;.
  * @param oOut File stream to write to.
  */
  void WriteAllometry(std::fstream &oOut);

};
//---------------------------------------------------------------------------
#endif // TestAdvancedHarvestShell_H
