//---------------------------------------------------------------------------
// TestNCIBAGrowth
//---------------------------------------------------------------------------
#if !defined(TestNCIBAGrowth_H)
  #define TestNCIBAGrowth_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* Tests the clNCIBAGrowth class.
*/
class clTestNCIBAGrowth : public CppUnit::TestFixture {

  public:

  /**
  * Tests the clNCIBAGrowth class.
  */
  void TestNCIBAGrowth();

  /**
  * Tests to make sure that the two NCI growth behaviors, diam only and auto
  * height, work correctly.  WriteXMLFile1() is used to do setup.  Then, this
  * will create two trees, one of each species.  Species 1 and Species 2 are
  * identical in every way.  Species 1 is assigned NCI growth as a
  * diameter-auto, and Species 2 is assigned NCI growth as diameter-only.
  * The two trees are assigned identical diameters and heights (heights that
  * do not match the diameters, allometrically-speaking). The same shade will
  * be assigned to each (0.5).  Growth will be allowed to occur, and then
  * this makes sure that the new diameters are identical but the heights
  * are not.
  * @param p_oSimManager Sim Manager object to do the runs.
  */
  void TestAutoHeightVsDiamOnly(clSimManager *p_oSimManager);

  /**
  * Performs a test run of NCI growth. WriteXMLFile2() performs setup for a
  * 1-year timestep.  There are 9 target trees whose values will be tested
  * according to the testing spreadsheet prepared.
  * @param p_oSimManager Sim Manager object to do the runs.
  */
  void TestRun1(clSimManager *p_oSimManager);

  /**
  * Performs a test run of NCI growth. WriteXMLFile3() performs setup for a
  * 3-year timestep.  There are 9 target trees whose values will be tested
  * according to the testing spreadsheet prepared.
  * @param p_oSimManager Sim Manager object to do the runs.
  */
  void TestRun2(clSimManager *p_oSimManager);
  
  /**
  * Performs a test run of NCI growth. WriteXMLFile4() performs setup for a
  * 1-year timestep.  There are 9 target trees whose values will be tested
  * according to the testing spreadsheet prepared.
  * @param p_oSimManager Sim Manager object to do the runs.
  */
  void TestRun3(clSimManager *p_oSimManager);

  /**
  * Tests error processing for input files.
  *
  * WriteXMLErrorFileX (where X = 1-5) is written and checked to make sure that
  * each produces an error.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  protected:

  /**
  * Writes a parameter file to direct testing.  This contains two species,
  * identical in all ways except Species 1 is assigned NCI growth
  * as a diameter-auto, and Species 2 is assigned NCI growth as diameter-only.
  *
  * @return Filename written.
  */
  char* WriteXMLFile1();

  /**
  * Writes a 1-year timestep file.  This contains 4 species.  NCI growth
  * (auto-height) is assigned to saplings and adults of Species 2, 3, and 4.
  *
  * @return Filename written.
  */
  char* WriteXMLFile2();

  /**
  * Writes a 3-year timestep file.  This contains 4 species.  NCI growth
  * (auto-height) is assigned to saplings and adults of Species 2, 3, and 4.
  *
  * @return Filename written.
  */
  char* WriteXMLFile3();
  
  /**
  * Writes a 1-year timestep file.  This contains 4 species.  NCI growth
  * (auto-height) is assigned to saplings and adults of Species 2, 3, and 4.
  *
  * @return Filename written.
  */
  char* WriteXMLFile4();

  /**
  * Writes an error condition where max radius of neighbor effects is < 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes an error condition where max growth for a species is < 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile2();

  /**
  * Writes an error condition where Xb for a species = 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile3();

  /**
  * Writes an error condition where X0 for a species = 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile4();

  /**
  * Writes an error condition where a neighbor DBH is less than 0.
  *
  * @return Filename written.
  */
  char* WriteXMLErrorFile5();

  /**
  * Writes common portions of the test parameter files (tree pop, plot, etc).
  * Picks up right after the number of years per timestep and finishes out the
  * file.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);
  
  /**
  * Writes common portions of the test parameter files (tree pop, plot, etc).
  * @param oOut File stream to write to.
  */
  void WriteTreeStuff(std::fstream &oOut);

  /**
  * Writes common portions of the test error parameter files (tree pop, plot,
  * etc).  Starts at the beginning of the file and ends at the closing tree tag.
  * @param oOut File stream to write to.
  */
  void WriteErrorFileCommonStuff(std::fstream &oOut);


};

//---------------------------------------------------------------------------
#endif // TestNCIBAGrowth_H
