//---------------------------------------------------------------------------
// TestNeighborhoodSeedPredation
//---------------------------------------------------------------------------
#if !defined(TestNeighborhoodSeedPredation_H)
  #define TestNeighborhoodSeedPredation_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;

/**
* This class tests the clNeighborhoodSeedPredation class.
*/
class clTestNeighborhoodSeedPredation : public CppUnit::TestFixture
{

  public:

  /**
  * Compiles all tests into one function.
  */
  void TestNeighborhoodSeedPredation();

  private:

  /**
  * First normal processing run. WriteXMLFile1() does setup. Seed predation is
  * applied to species 1, 2, and 4. This run tests not counting seeds of
  * unapplied species when checking the masting threshold, and choosing the
  * correct parameters in a non-mast situation. This uses non-spatial disperse
  * to drop 1 seed/m2 of each species, plus 2 of species 3 (5 seeds/m2 total).
  * After a run occurs, the number of new seedlings is verified.
  * @param p_oSimManager Sim manager object.
  */
  void TestNormalProcessingRun1(clSimManager *p_oSimManager);

  /**
  * Second normal processing run. WriteXMLFile2() does setup. This tests
  * choosing the correct parameters in a mast situation, and correctly
  * calculating mast density. This puts 10000 seeds of species 1 in seed grid
  * cell (0,0) and 100 seeds of species 2 in grid cell (4,4). After a run
  * occurs, the number of new seedlings is verified.
  * @param p_oSimManager Sim manager object.
  */
  void TestNormalProcessingRun2(clSimManager *p_oSimManager);

  /**
  * Third normal processing run. WriteXMLFile3() does setup. Seed predation is
  * applied to species 1, 2, and 4. This run tests multi-year timesteps. This
  * uses non-spatial disperse to drop 3 seeds/m2 of each species, plus 2 of
  * species 3 (11 seeds/m2 total). After a run occurs, the number of new
  * seedlings is verified.
  * @param p_oSimManager Sim manager object.
  */
  void TestNormalProcessingRun3(clSimManager *p_oSimManager);

  /**
  * Fourth normal processing run. WriteXMLFile4() does setup. Linked seed
  * predation is applied to species 1, 2, and 4, and unlinked to species 5.
  * This run tests linked vs. standalone predation. After a run occurs, the
  * number of new seedlings is verified.
  * @param p_oSimManager Sim manager object.
  */
  void TestNormalProcessingRun4(clSimManager *p_oSimManager);

  /**
   * Fifth normal processing run.  WriteXMLFile5() does setup.  Clone of run 1,
   * except the method of determining masting is asking the disperse behaviors.
   * @param p_oSimManager Sim manager object.
   */
  void TestNormalProcessingRun5(clSimManager *p_oSimManager);

  /**
   * Sixth normal processing run.  WriteXMLFile6 () does setup.  Clone of run 2,
   * except the method of determining masting is asking the disperse behaviors.
   * @param p_oSimManager Sim manager object.
   */
  void TestNormalProcessingRun6(clSimManager *p_oSimManager);

  /**
   * Seventh normal processing run. WriteXMLFile7() does setup. Clone of run 2,
   * except the method of determining masting is asking the disperse behaviors.
   * @param p_oSimManager Sim manager object.
   */
  void TestNormalProcessingRun7(clSimManager *p_oSimManager);

  /**
  * Tests error processing. This uses the error parameter files to make sure
  * errors are correctly caught.
  * @param p_oSimManager Sim manager object.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
   * Does the testing for runs one and five
   * @param p_oSimManager Sim manager object.
   */
  void TestRun1And5(clSimManager *p_oSimManager);

  /**
   * Does the testing for runs two and six
   * @param p_oSimManager Sim manager object.
   */
  void TestRun2And6(clSimManager *p_oSimManager);

  /**
  * Writes the parameter file for normal processing run 1.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes the parameter file for normal processing run 2.
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes the parameter file for normal processing run 3.
  * @return Filename written.
  */
  char * WriteXMLFile3();

  /**
  * Writes the parameter file for normal processing run 4.
  * @return Filename written.
  */
  char * WriteXMLFile4();

  /**
  * Writes the parameter file for normal processing run 5.
  * @return Filename written.
  */
  char * WriteXMLFile5();

  /**
  * Writes the parameter file for normal processing run 6.
  * @return Filename written.
  */
  char * WriteXMLFile6();

  /**
  * Writes the parameter file for normal processing run 7.
  * @return Filename written.
  */
  char * WriteXMLFile7();

  /**
   * Writes a file where minimum neighbor DBH is negative.
   * @return Filename written.
   */
  char* WriteXMLErrorFile1();

  /**
   * Writes a file where masting threshod is negative.
   * @return Filename written.
   */
  char* WriteXMLErrorFile2();

  /**
   * Writes a file where neighborhood radius is negative.
   * @return Filename written.
   */
  char* WriteXMLErrorFile3();

  /**
   * Writes a file where there are no disperse behaviors.
   * @return Filename written.
   */
  char* WriteXMLErrorFile4();

  /**
   * Writes a file where the neighborhood seed predation behavior is the linked
   * version but there is no functional response seed predation.
   * @return Filename written.
   */
  char* WriteXMLErrorFile5();

  /** Writes common portions of the parameter file - trees and allometry.
   * @param oOut Filestream to write to.
   */
  void WriteTreesAllometry(std::fstream &oOut);

  /**
   * Writes common seed predation parameters
   * @param oOut File stream to write to.
   */
  void WriteCommonPredationParameters (std::fstream &oOut);
};
//---------------------------------------------------------------------------

#endif // TestNeighborhoodSeedPredation_H
