//---------------------------------------------------------------------------
// TestSnagDecomp
//---------------------------------------------------------------------------
#if !defined(TestSnagDecomp_H)
  #define TestSnagDecomp_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>


class clSimManager;
class clTreePopulation;
class clTree;

/**
* Tests the SnagDecomp class descended from clBehaviorBase.
* @author Mark Vanderwel
*/
class clTestSnagDecomp : public CppUnit::TestFixture
{
  public:

  /**
  * Tests the snag decay class dynamics class.
  * WriteXMLFile1() and WriteXMLFile2() create separate parameter file.  
  * The files have the following common characteristics:
  * <ul>
  * <li>Timestep length of 5 years.
  * <li>1 species used: ACSA
  * </ul>
  * WriteXMLFile1() tests the snag fall function under harvesting, as follows:
  * <ul>
  * <li>22500 live trees and 5600 decay class 1 snags are created.
  * <li>The model is run for 1 timestep.
  * <li>5 m2/ha is cut roughly evenly across grid cells.  The dead tree remover 
  * behavior is not invoked so they do not become snags.
  * <li>The number of snags in each decay class is counted and compared to expected 
  * results.  Individual decay classes are +/- 10%, and total snags are +/- 2%. 
  * </ul>
  * WriteXMLFile2() tests the tree fall function, as follows:
  * <ul>
  * <li>11200 live trees are created.
  * <li>The model is run for 1 timestep.
  * <li>100% mortality kills all these trees.
  * <li>The number of snags in each decay class is counted and compared to expected 
  * results.  Individual decay classes are +/- 10%, and total snags are +/- 2%. 
  * </ul>
  * <p>
  * <p>
  * Error condition 1: WriteXMLErrorFile1() creates the paramter file.  The
  * behavior finds a negative parameter value and correctly throws an error.
  * <p>
  * Error condition 2: WriteXMLErrorFile2() creates the parameter file.  The
  * behaviour finds that the sum of transition probabilities does not add up to
  * 1 and correctly throws an error.
  */


  void TestSnagDecomp();

  protected:


/**
* Tests error processing.  This uses WriteXMLErrorFileX() to introduce errors
* and makes sure that they are caught.
*/

  void TestErrorProcessing (clSimManager *p_oSimManager);

  /**
  * Writes a parameter file with the specifications given for
  * TestSnagDecomp(), run 1.
  * @return Filename written.
  */
  char * WriteXMLFile1();
  
  
    /**
  * Writes a parameter file with the specifications given for
  * TestSnagDecomp(), run 2.
  * @return Filename written.
  */
  char * WriteXMLFile2();


/**
* Writes a parameter file with the specifications given for
* TestSnagDecomp(), error condition 1.
* @return Filename written.
*/

  char * WriteXMLErrorFile1();

  /**
* Writes a parameter file with the specifications given for
* TestSnagDecomp(), error condition 2.
* @return Filename written.
*/

  char * WriteXMLErrorFile2();

};
//---------------------------------------------------------------------------
#endif // TestSnagDecomp_H
