//---------------------------------------------------------------------------
// TestStorm
//---------------------------------------------------------------------------
#if !defined(TestStorm_H)
  #define TestStorm_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clSimManager;

/**
* Tests the clStorm class.
*/
class clTestStorm : public CppUnit::TestFixture {

  public:

  /**
  * Does all the storm testing.
  */
  void TestStorm();

  protected:

  /**
  * Tests normal processing for a deterministic damage pattern.  WriteXMLFile1()
  * does setup.  Here's how it goes:
  * <ul>
  * <li>"Storm Damage" Grid:  10 X 10 cells.</li>
  * <li>Storm return intervals:
  * <ul>
  *   <li>1 - 0</li>
  *   <li>2 - 1</li>
  *   <li>3 - 20</li>
  *   <li>4 - 1.00E+28</li>
  *   <li>5 - 13.2</li>
  *   <li>6 - 45</li>
  *   <li>7 - 50</li>
  *   <li>8 - 55</li>
  *   <li>9 - 60</li>
  *   <li>10 - 65</li>
  * </ul></li>
  * <li>Pre-run:  Make sure probabilities are calculated correctly.
  * Time since last storm = 0.</li>
  * <li>Timestep 1:  No storms.  All cell values = 0.  Time since
  * last storm = 1.</li>
  * <li>Timestep 2:  1 storm of severity 0.4-0.5.  All cell values between 0.4
  * and 0.5.  Time since last storm = 0.</li>
  * <li>Timestep 3:  2 storms of severity 0.2-0.3 and 0.5-0.6.  All cell values
  * between 0.7 and 0.9.  Time since last storm = 0.</li>
  * <li>Timestep 4:  No storms.  All cell values = 0.  Time since last
  * storm = 1.</li>
  * <li>Timestep 5:  2 storms of of severity 0.6-0.7 and 0.8-0.9.  All cell
  * values = 1.  Time since last storm = 0.</li>
  * <li>Timesteps 6, 7, 8 No storms.  All cell values 0.  Time since last
  * storm = 1, 2, 3.</li>
  * </ul>
  * @param p_oSimManager Sim Manager object.
  */
  void TestNormalDeterministic(clSimManager *p_oSimManager);

  /**
  * Tests normal processing for a deterministic mapped damage pattern.
  * WriteXMLFile2() does setup.  Here's how it goes:
  * <ul>
  * <li>"Storm Damage" Grid:   5 X 3 cells.  Susceptibility map values are
  * X/10 + Y/100.</li>
  * <li>Timestep 1:  1 storm of severity 0.1-0.2.  Cell values between
  * (x/10 + y/100) * 0.1 and (x/10 + y/100) * 0.2.</li>
  * <li>Timestep 2:  3 storms of 0-0.1, 0.1-0.2, and 0.2-0.3.  Cell values
  * between (x/10 + y/100) * 0.3 and (x/10 + y/100) * 0.6.</li>
  * <li>Timestep 3:  No storms.  All cell values = 0.</li>
  * <li>Timestep 4:  2 storms of severity 0.6-0.7 and 0.9 - 1.  All cell
  * values between x/10 + y/100 * 1.5 and 1.</li>
  * </ul>
  * @param p_oSimManager Sim Manager object.
  */
  void TestNormalMappedDeterministic(clSimManager *p_oSimManager);

  /**
  * Tests normal processing for a stochastic damage pattern.  WriteXMLFile3()
  * does setup.  Here's how it goes:
  * <ul>
  * <li>Grid resolution is 10X10.</li>
  * <li>Normal-distribute the results with a standard deviation of 0.1.</li>
  * <li>Storm damage indexes are the same as for
  * TestNormalDeterministic().</li>
  * <li>Timestep 1:  No storms.  All cell values = 0.</li>
  * <li>Timestep 2:  2 storms of severity 0.6-0.7 and 0.8-0.9.  Cell values
  * have a mean of 1 and a max of 1.</li>
  * <li>Timestep 3:  1 storm of severity 0.4 - 0.5.  Cell values have a mean of
  * between 0.4 and 0.5.</li>
  * </ul>
  * @param p_oSimManager Sim Manager object.
  */
  void TestNormalStochastic(clSimManager *p_oSimManager);

  /**
  * Tests normal processing for a stochastic mapped damage pattern.
  * WriteXMLFile4() does setup.  Here's how it goes:
  * <ul>
  * <li>"Storm Damage" Grid:   5 X 3 cells.  Susceptibility map values are
  * X/10 + Y/100.</li>
  * <li>Timestep 1:  2 storms of severity 0.7-0.8 and 0.8-0.9.  Cell values
  * between (x/10+y/100)*1.5 and 1.</li>
  * <li>Timestep 2:  1 storm of severity 0.3-0.4.  Cell values between
  * (x/10+y/100)*0.3 and (x/10+y/100)*0.4.</li>
  * <li>Timestep 3:  No storms.  All cell values = 0.</li>
  * </ul>
  */
  void TestNormalMappedStochastic(clSimManager *p_oSimManager);

  /**
  * Tests multi-year timestep testing.  WriteXMLFile5() does setup.  Here's how
  * it goes:
  * <ul>
  * <li>Timestep length is 2 years.</li>
  * <li>Default grid cell lengths.</li>
  * <li>Timestep 1:  2 storm of severity 0.2-0.3.  All cell values between
  * 0.4 and 0.6. Time since last storm = 0.</li>
  * <li>Timestep 2:  No storms.  All cell values = 0.  Time since last 
  * storm = 2.</li>
  * <li>Timestep 3:  1 storm.  Time since last storm = 0.</li>
  * <li>Timestep 4:  No storms.  All cell values = 0.  Time since last
  * hurricane = 2.</li>
  * <li>Timestep 5:  No storms.  All cell values = 0.  Time since last
  * hurricane = 4.</li>
  * </ul>
  */
  void TestMultiYearTimesteps(clSimManager *p_oSimManager);

  /**
  * Tests error processing.  Tests:
  * <ul>
  * <li>There are grid maps for both "Storm Damage" and "Storm Susceptibility"
  * and the grid cell resolutions don't match.</li>
  * <li>The damage pattern is set to "mapped" but there is no "Storm
  * Susceptibility" grid map in the parameter file.</li>
  * <li>The damage pattern is set to "mapped", there is a grid map for "Storm
  * Susceptibility" in the parameter file, but not all values are between 0
  * and 1.</li>
  * <li>Not all storm interval return values are between 0 and 1.</li>
  * <li>The value for storm damage pattern in the parameter file is not
  * recognized.</li>
  * <li>The damage pattern is "stochastic" and there is no value for
  * probability distribution function.</li>
  * <li>The damage pattern is "stochastic" and the value for probability
  * distribution function is not recognized.</li>
  * <li>The damage pattern is "stochastic", the probability distribution
  * function is "normal" or "lognormal", and there is no value for standard
  * deviation.</li>
  * <li>The damage pattern is "stochastic", the probability distribution
  * function is "negative_binomial", and there is no value for the clumping
  * parameter.</li>
  * </ul>
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file with the specifications given for
  * TestNormalDeterministic().
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file with the specifications given for
  * TestNormalMappedDeterministic().
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file with the specifications given for
  * TestNormalStochastic().
  * @return Filename written.
  */
  char * WriteXMLFile3();

  /**
  * Writes a parameter file with the specifications given for
  * TestNormalMappedStochastic().
  * @return Filename written.
  */
  char * WriteXMLFile4();

  /**
  * Writes a parameter file with the specifications given for
  * TestMultiYearTimesteps().
  * @return Filename written.
  */
  char * WriteXMLFile5();

  /**
  * Writes a parameter file where there are grid maps for both "Storm Damage"
  * and "Storm Susceptibility" and the grid cell resolutions don't match.
  * @return Filename written.
  */
  char * WriteXMLErrorFile1();

  /**
  * Writes a parameter file where the damage pattern is set to "mapped" but
  * there is no "Storm Susceptibility" grid map in the parameter file.
  * @return Filename written.
  */
  char * WriteXMLErrorFile2();

  /**
  * Writes a parameter file where not all storm interval return values are
  * greater than 0.
  * @return Filename written.
  */
  char * WriteXMLErrorFile3();

  /**
  * Writes a parameter file where the value for susceptibility pattern in the
  * parameter file is not recognized.
  * @return Filename written.
  */
  char * WriteXMLErrorFile4();

  /**
  * Writes a parameter file where the damage pattern is "stochastic" and there
  * is no value for probability distribution function.
  * @return Filename written.
  */
  char * WriteXMLErrorFile5();

  /**
  * Writes a parameter file where the damage pattern is "stochastic" and the
  * value for probability distribution function is not recognized.
  * @return Filename written.
  */
  char * WriteXMLErrorFile6();

  /**
  * Writes a parameter file where the damage pattern is "stochastic", the
  * probability distribution function is "normal" or "lognormal", and there is
  * no value for standard deviation.
  * @return Filename written.
  */
  char * WriteXMLErrorFile7();

  /**
  * Writes a parameter file where the value for stochasticity is not recognized.
  * @return Filename written.
  */
  char * WriteXMLErrorFile8();
  
  /**
  * Writes a parameter file where a scheduled storm's minimum is greater than
  * its maximum.
  * @return Filename written.
  */
  char * WriteXMLErrorFile9();
  
  /**
  * Writes a parameter file where a scheduled storm's minimum is not between
  * 0 and 1.
  * @return Filename written.
  */
  char * WriteXMLErrorFile10();
  
  /**
  * Writes a parameter file where a scheduled storm's maximum is not between
  * 0 and 1.
  * @return Filename written.
  */
  char * WriteXMLErrorFile11();

  /**
  * Writes the portion of the parameter file common to all test files.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);

};
//---------------------------------------------------------------------------
#endif // TestStorm_H
